@@ -20,50 +20,40 @@
 
 /**
  * SECTION:element-theoraparse
- * @short_description: parses theora streams 
  * @see_also: theoradec, oggdemux, vorbisparse
  *
- * <refsect2>
- * <para>
  * The theoraparse element will parse the header packets of the Theora
  * stream and put them as the streamheader in the caps. This is used in the
  * multifdsink case where you want to stream live theora streams to multiple
  * clients, each client has to receive the streamheaders first before they can
  * consume the theora packets.
- * </para>
- * <para>
+ *
  * This element also makes sure that the buffers that it pushes out are properly
  * timestamped and that their offset and offset_end are set. The buffers that
- * vorbisparse outputs have all of the metadata that oggmux expects to receive,
+ * theoraparse outputs have all of the metadata that oggmux expects to receive,
  * which allows you to (for example) remux an ogg/theora file.
- * </para>
- * <para>
+ *
  * In addition, this element allows you to fix badly synchronized streams. You
  * pass in an array of (granule time, buffer time) synchronization points via
  * the synchronization-points GValueArray property, and this element will adjust
  * the granulepos values that it outputs. The adjustment will be made by
  * offsetting all buffers that it outputs by a specified amount, and updating
  * that offset from the value array whenever a keyframe is processed.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=video.ogg ! oggdemux ! theoraparse ! fakesink
- * </programlisting>
- * This pipeline shows that the streamheader is set in the caps, and that each
+ * ]| This pipeline shows that the streamheader is set in the caps, and that each
  * buffer has the timestamp, duration, offset, and offset_end set.
- * </para>
- * <para>
- * <programlisting>
- * gst-launch filesrc location=video.ogg ! oggdemux ! vorbisparse \
+ * |[
+ * gst-launch filesrc location=video.ogg ! oggdemux ! theoraparse \
  *            ! oggmux ! filesink location=video-remuxed.ogg
- * </programlisting>
- * This pipeline shows remuxing. video-remuxed.ogg might not be exactly the same
+ * ]| This pipeline shows remuxing. video-remuxed.ogg might not be exactly the same
  * as video.ogg, but they should produce exactly the same decoded data.
- * </para>
  * </refsect2>
  *
- * Last reviewed on 2006-04-01 (0.10.4.1)
+ * Last reviewed on 2008-05-28 (0.10.20)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,7 +66,7 @@
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 static GstElementDetails theora_parse_details = {
-  "TheoraParse",
+  "Theora video parser",
   "Codec/Parser/Video",
   "parse raw theora streams",
   "Andy Wingo <wingo@pobox.com>"
@@ -153,7 +143,8 @@ gst_theora_parse_class_init (GstTheoraParseClass * klass)
           "An array of (granuletime, buffertime) pairs",
           g_param_spec_uint64 ("time", "Time",
               "Time (either granuletime or buffertime)", 0, G_MAXUINT64, 0,
-              G_PARAM_READWRITE), (GParamFlags) G_PARAM_READWRITE));
+              G_PARAM_READWRITE),
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = theora_parse_change_state;
 
@@ -279,7 +270,9 @@ theora_parse_set_header_on_caps (GstTheoraParse * parse, GstCaps * caps)
   g_value_init (&array, GST_TYPE_ARRAY);
 
   for (i = 0; i < 3; i++) {
-    g_assert (bufs[i]);
+    if (bufs[i] == NULL)
+      continue;
+
     bufs[i] = gst_buffer_make_metadata_writable (bufs[i]);
     GST_BUFFER_FLAG_SET (bufs[i], GST_BUFFER_FLAG_IN_CAPS);
 
@@ -313,6 +306,7 @@ theora_parse_set_streamheader (GstTheoraParse * parse)
 {
   GstCaps *caps;
   gint i;
+  guint32 bitstream_version;
 
   g_assert (!parse->streamheader_received);
 
@@ -325,8 +319,12 @@ theora_parse_set_streamheader (GstTheoraParse * parse)
   for (i = 0; i < 3; i++) {
     ogg_packet packet;
     GstBuffer *buf;
+    int ret;
 
     buf = parse->streamheader[i];
+    if (buf == NULL)
+      continue;
+
     gst_buffer_set_caps (buf, GST_PAD_CAPS (parse->srcpad));
 
     packet.packet = GST_BUFFER_DATA (buf);
@@ -334,13 +332,28 @@ theora_parse_set_streamheader (GstTheoraParse * parse)
     packet.granulepos = GST_BUFFER_OFFSET_END (buf);
     packet.packetno = i + 1;
     packet.e_o_s = 0;
-    theora_decode_header (&parse->info, &parse->comment, &packet);
+    packet.b_o_s = (i == 0);
+    ret = theora_decode_header (&parse->info, &parse->comment, &packet);
+    if (ret < 0) {
+      GST_WARNING_OBJECT (parse, "Failed to decode Theora header %d: %d\n",
+          i + 1, ret);
+    }
   }
 
   parse->fps_n = parse->info.fps_numerator;
   parse->fps_d = parse->info.fps_denominator;
   parse->shift = _theora_ilog (parse->info.keyframe_frequency_force - 1);
 
+  /* With libtheora-1.0beta1 the granulepos scheme was changed:
+   * where earlier the granulepos refered to the index/beginning
+   * of a frame, it now refers to the end, which matches the use
+   * in vorbis/speex. We check the bitstream version from the header so
+   * we know which way to interpret the incoming granuepos
+   */
+  bitstream_version = (parse->info.version_major << 16) |
+      (parse->info.version_minor << 8) | parse->info.version_subminor;
+  parse->is_old_bitstream = (bitstream_version <= 0x00030200);
+
   parse->streamheader_received = TRUE;
 }
 
@@ -367,10 +380,12 @@ theora_parse_push_headers (GstTheoraParse * parse)
 
   /* ignore return values, we pass along the result of pushing data packets only
    */
-  for (i = 0; i < 3; i++)
-    gst_pad_push (parse->srcpad, gst_buffer_ref (parse->streamheader[i]));
+  for (i = 0; i < 3; i++) {
+    GstBuffer *buf;
 
-  parse->send_streamheader = FALSE;
+    if ((buf = parse->streamheader[i]))
+      gst_pad_push (parse->srcpad, gst_buffer_ref (buf));
+  }
 }
 
 static void
@@ -386,33 +401,48 @@ theora_parse_clear_queue (GstTheoraParse * parse)
     GstEvent *event;
 
     event = GST_EVENT_CAST (g_queue_pop_head (parse->event_queue));
-    gst_buffer_unref (event);
+    gst_event_unref (event);
   }
 }
 
 static gint64
-make_granulepos (gint64 keyframe, gint64 frame, gint shift)
+make_granulepos (GstTheoraParse * parse, gint64 keyframe, gint64 frame)
 {
+  gint64 iframe;
+
   if (keyframe == -1)
     keyframe = 0;
+  /* If using newer theora, offset the granulepos by +1, see comment in
+   * theora_parse_set_streamheader.
+   * 
+   * We don't increment keyframe directly, as internally we always index frames
+   * starting from 0 and we do some sanity checking below. */
+  if (!parse->is_old_bitstream)
+    iframe = keyframe + 1;
+  else
+    iframe = keyframe;
 
   g_return_val_if_fail (frame >= keyframe, -1);
-  g_return_val_if_fail (frame - keyframe < 1 << shift, -1);
+  g_return_val_if_fail (frame - keyframe < 1 << parse->shift, -1);
 
-  return (keyframe << shift) + (frame - keyframe);
+  return (iframe << parse->shift) + (frame - keyframe);
 }
 
 static void
-parse_granulepos (gint64 granulepos, gint shift, gint64 * keyframe,
-    gint64 * frame)
+parse_granulepos (GstTheoraParse * parse, gint64 granulepos,
+    gint64 * keyframe, gint64 * frame)
 {
   gint64 kf;
 
-  kf = granulepos >> shift;
+  kf = granulepos >> parse->shift;
+  /* If using newer theora, offset the granulepos by -1, see comment
+   * in theora_parse_set_streamheader */
+  if (!parse->is_old_bitstream)
+    kf -= 1;
   if (keyframe)
     *keyframe = kf;
   if (frame)
-    *frame = kf + (granulepos & ((1 << shift) - 1));
+    *frame = kf + (granulepos & ((1 << parse->shift) - 1));
 }
 
 static gboolean
@@ -474,7 +504,7 @@ theora_parse_push_buffer (GstTheoraParse * parse, GstBuffer * buf,
   next_time = gst_util_uint64_scale_int (GST_SECOND * (frame + 1),
       parse->fps_d, parse->fps_n);
 
-  GST_BUFFER_OFFSET_END (buf) = make_granulepos (keyframe, frame, parse->shift);
+  GST_BUFFER_OFFSET_END (buf) = make_granulepos (parse, keyframe, frame);
   GST_BUFFER_OFFSET (buf) = this_time;
   GST_BUFFER_TIMESTAMP (buf) = this_time;
   GST_BUFFER_DURATION (buf) = next_time - this_time;
@@ -521,7 +551,7 @@ theora_parse_drain_queue_prematurely (GstTheoraParse * parse)
 
     if (parse->prev_keyframe < 0) {
       if (GST_BUFFER_OFFSET_END_IS_VALID (buf)) {
-        parse_granulepos (GST_BUFFER_OFFSET_END (buf), parse->shift,
+        parse_granulepos (parse, GST_BUFFER_OFFSET_END (buf),
             &parse->prev_keyframe, NULL);
       } else {
         /* No previous keyframe known; can't extract one from this frame. That
@@ -550,9 +580,20 @@ theora_parse_drain_queue (GstTheoraParse * parse, gint64 granulepos)
   GstFlowReturn ret = GST_FLOW_OK;
   gint64 keyframe, prev_frame, frame;
 
-  parse_granulepos (granulepos, parse->shift, &keyframe, &frame);
+  parse_granulepos (parse, granulepos, &keyframe, &frame);
+
+  GST_DEBUG ("draining queue of length %d",
+      g_queue_get_length (parse->buffer_queue));
+
+  GST_LOG_OBJECT (parse, "gp %" G_GINT64_FORMAT ", kf %" G_GINT64_FORMAT
+      ", frame %" G_GINT64_FORMAT, granulepos, keyframe, frame);
 
   prev_frame = frame - g_queue_get_length (parse->buffer_queue);
+
+  GST_LOG_OBJECT (parse,
+      "new prev %" G_GINT64_FORMAT ", prev %" G_GINT64_FORMAT, prev_frame,
+      parse->prev_frame);
+
   if (prev_frame < parse->prev_frame) {
     GST_WARNING ("jumped %" G_GINT64_FORMAT
         " frames backwards! not sure what to do here",
@@ -567,9 +608,6 @@ theora_parse_drain_queue (GstTheoraParse * parse, gint64 granulepos)
     parse->prev_frame = prev_frame;
   }
 
-  GST_DEBUG ("draining queue of length %d",
-      g_queue_get_length (parse->buffer_queue));
-
   while (!g_queue_is_empty (parse->buffer_queue)) {
     GstBuffer *buf;
 
@@ -606,7 +644,7 @@ theora_parse_queue_buffer (GstTheoraParse * parse, GstBuffer * buf)
 
   if (GST_BUFFER_OFFSET_END_IS_VALID (buf)) {
     if (parse->prev_keyframe < 0) {
-      parse_granulepos (GST_BUFFER_OFFSET_END (buf), parse->shift,
+      parse_granulepos (parse, GST_BUFFER_OFFSET_END (buf),
           &parse->prev_keyframe, NULL);
     }
     ret = theora_parse_drain_queue (parse, GST_BUFFER_OFFSET_END (buf));
@@ -619,24 +657,38 @@ static GstFlowReturn
 theora_parse_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstFlowReturn ret;
-  GstBuffer *buf;
   GstTheoraParse *parse;
+  guint8 *data;
+  guint size;
+  gboolean have_header;
 
   parse = GST_THEORA_PARSE (gst_pad_get_parent (pad));
 
-  buf = GST_BUFFER (buffer);
-  parse->packetno++;
+  data = GST_BUFFER_DATA (buffer);
+  size = GST_BUFFER_SIZE (buffer);
 
-  if (parse->packetno <= 3) {
-    /* if 1 <= packetno <= 3, it's streamheader,
-     * so put it on the streamheader list and return */
-    parse->streamheader[parse->packetno - 1] = buf;
+  have_header = FALSE;
+  if (size >= 1) {
+    if (data[0] & 0x80)
+      have_header = TRUE;
+  }
+
+  if (have_header) {
+    if (parse->send_streamheader) {
+      /* we need to collect the headers still */
+      /* so put it on the streamheader list and return */
+      if (data[0] >= 0x80 && data[0] <= 0x82)
+        parse->streamheader[data[0] - 0x80] = buffer;
+    }
     ret = GST_FLOW_OK;
   } else {
-    if (parse->send_streamheader)
+    /* data packet, push the headers we collected before */
+    if (parse->send_streamheader) {
       theora_parse_push_headers (parse);
+      parse->send_streamheader = FALSE;
+    }
 
-    ret = theora_parse_queue_buffer (parse, buf);
+    ret = theora_parse_queue_buffer (parse, buffer);
   }
 
   gst_object_unref (parse);
@@ -860,7 +912,6 @@ theora_parse_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       theora_info_init (&parse->info);
       theora_comment_init (&parse->comment);
-      parse->packetno = 0;
       parse->send_streamheader = TRUE;
       parse->buffer_queue = g_queue_new ();
       parse->event_queue = g_queue_new ();
